@@ -2,7 +2,7 @@ package Module::Install::XSUtil;
 
 use 5.005_03;
 
-$VERSION = '0.23';
+$VERSION = '0.24';
 
 use Module::Install::Base;
 @ISA     = qw(Module::Install::Base);
@@ -85,18 +85,26 @@ sub _is_msvc{
     my $cc_available;
 
     sub cc_available {
-        return $cc_available if defined $cc_available;
+        return defined $cc_available ?
+            $cc_available :
+            ($cc_available = shift->can_cc())
+        ;
+    }
+
+    my $want_xs;
+    sub want_xs {
+        my $default = @_ ? shift : 1; # you're using this module, you /must/ want XS by default
+        return $want_xs if defined $want_xs;
 
         foreach my $arg(@ARGV){
             if($arg eq '--pp'){
-                return $cc_available = 0;
+                return $want_xs = 0;
             }
             elsif($arg eq '--xs'){
-                return $cc_available = 1;
+                return $want_xs = 1;
             }
         }
-
-        return $cc_available = shift->can_cc();
+        return $want_xs = $default;
     }
 }
 
@@ -171,7 +179,6 @@ sub cc_append_to_inc{
     for my $dir(@dirs){
         unless(-d $dir){
             warn("'$dir' not found: $!\n");
-            exit;
         }
 
         _verbose "inc: -I$dir" if _VERBOSE;
@@ -189,19 +196,10 @@ sub cc_append_to_inc{
     return;
 }
 
-
 sub cc_libs {
-    goto &cc_append_to_libs;
-}
+    my ($self, @libs) = @_;
 
-sub cc_append_to_libs{
-    my($self, @libs) = @_;
-
-    $self->_xs_initialize();
-
-    my $mm = $self->makemaker_args;
-
-    my $libs = join q{ }, map{
+    @libs = map{
         my($name, $dir) = ref($_) eq 'ARRAY' ? @{$_} : ($_, undef);
         my $lib;
         if(defined $dir) {
@@ -210,12 +208,26 @@ sub cc_append_to_libs{
         else {
             $lib = '';
         }
-        $lib .= ($name =~ /^-/ ? qq{$name } : qq{-l$name});
+        $lib .= ($name =~ /^-/ ? qq{$name} : qq{-l$name});
         _verbose "libs: $lib" if _VERBOSE;
         $lib;
     } @libs;
 
-    if($mm->{LIBS}){
+    $self->cc_append_to_libs( @libs );
+}
+
+sub cc_append_to_libs{
+    my($self, @libs) = @_;
+
+    $self->_xs_initialize();
+
+    return unless @libs;
+
+    my $libs = join q{ }, @libs;
+
+    my $mm = $self->makemaker_args;
+
+    if ($mm->{LIBS}){
         $mm->{LIBS} .= q{ } . $libs;
     }
     else{
@@ -224,6 +236,32 @@ sub cc_append_to_libs{
     return $libs;
 }
 
+sub cc_assert_lib {
+    my ($self, @dcl_args) = @_;
+
+    if ( ! $self->{xsu_loaded_checklib} ) {
+        my $loaded_lib = 0;
+        foreach my $checklib qw(inc::Devel::CheckLib Devel::CheckLib) {
+            eval "use $checklib 0.4";
+            if (!$@) {
+                $loaded_lib = 1;
+                last;
+            }
+        }
+
+        if (! $loaded_lib) {
+            warn "Devel::CheckLib not found in inc/ nor \@INC";
+            exit 0;
+        }
+
+        $self->{xsu_loaded_checklib}++;
+        $self->configure_requires( "Devel::CheckLib" => "0.4" );
+        $self->build_requires( "Devel::CheckLib" => "0.4" );
+    }
+
+    Devel::CheckLib::check_lib_or_exit(@dcl_args);
+}
+
 sub cc_append_to_ccflags{
     my($self, @ccflags) = @_;
 
@@ -551,7 +589,7 @@ Module::Install::XSUtil - Utility functions for XS modules
 
 =head1 VERSION
 
-This document describes Module::Install::XSUtil version 0.23.
+This document describes Module::Install::XSUtil version 0.24.
 
 =head1 SYNOPSIS
 
@@ -595,11 +633,21 @@ See L<XS::MRO::Compat> and L<Method::Cumulative> for example.
 
 =head2 cc_available
 
-Returns true if a C compiler is available. If one passes C<--xs> to
-F<Makefile.PL>, this command returns true, and If one passes C<--pp>,
-it returns false.
+Returns true if a C compiler is available. YOU DO NOT NEED TO CALL
+THIS FUNCTION YOURSELF: it will be called for you when this module is 
+initialized, and your Makefile.PL process will exit with 0 status.
+Only explicitly call if you need to do some esoteric handling when
+no compiler is available (for example, when you have a pure perl alternative)
 
-This uses C<ExtUtils::CBuilder>.
+=head2 want_xs ?$default
+
+Returns true if the user asked for the XS version or pure perl version of the
+module.
+
+Will return true if C<--xs> is explicitly specified as the argument to 
+F<Makefile.PL>, and false if C<--pp> is specified. If neither is explicitly
+specified, will return the value specified by C<$default>. If you do not
+specify the value of C<$default>, then it will be true.
 
 =head2 requires_xs $module => ?$version
 
@@ -640,6 +688,12 @@ e.g.:
     cc_libs  'foo'; # ditto.
     cc_libs qw(-L/path/to/libs foo bar); # with library paths
 
+=head2 cc_assert_lib %args
+
+Checks if the given C library is installed via Devel::CheckLib. 
+Takes exactly what Devel::CheckLib takes. Note that you must pass
+the path names explicitly.
+
 =head2 install_headers ?@header_files
 
 Declares providing header files, extracts functions from these header files,
@@ -688,6 +742,12 @@ No bugs have been reported.
 
 Please report any bugs or feature requests to the author.
 
+=head1 ACKNOWLEDGEMENT
+
+Thanks to Taro Nishino for the test reports.
+
+Tanks to lestrrat for the suggestions and patches.
+
 =head1 AUTHOR
 
 Goro Fuji (gfx) E<lt>gfuji(at)cpan.orgE<gt>.
